@@ -1,8 +1,8 @@
 # $File: //member/autrijus/PAR/lib/PAR.pm $ $Author: autrijus $
-# $Revision: #22 $ $Change: 7159 $ $DateTime: 2003/07/27 14:05:08 $ vim: expandtab shiftwidth=4
+# $Revision: #41 $ $Change: 8540 $ $DateTime: 2003/10/23 04:42:43 $ vim: expandtab shiftwidth=4
 
 package PAR;
-$PAR::VERSION = '0.69_93';
+$PAR::VERSION = '0.75_99';
 
 use 5.006;
 use strict;
@@ -15,7 +15,7 @@ PAR - Perl Archive Toolkit
 
 =head1 VERSION
 
-This document describes version 0.69_93 of PAR, released July 27, 2003.
+This document describes version 0.75_99 of PAR, released October 23, 2003.
 
 =head1 SYNOPSIS
 
@@ -98,12 +98,12 @@ under F<arch/> and F<lib/>, e.g.:
 Afterward, you can just use F<mymodule.par> anywhere in your C<@INC>,
 use B<PAR>, and it will Just Work.
 
-For maximal convenience, you can set the C<PERL5OPT> environment
-variable to C<-MPAR> to enable C<PAR> processing globally (the overhead
-is small if not used), or to C<-MPAR=/path/to/mylib.par> to load a
+For convenience, you can set the C<PERL5OPT> environment variable to
+C<-MPAR> to enable C<PAR> processing globally (the overhead is small
+if not used); setting it to C<-MPAR=/path/to/mylib.par> will load a
 specific PAR file.  Alternatively, consider using the F<par.pl> utility
-bundled with this module, or the self-contained F<parl> utility on
-machines without PAR.pm installed.
+bundled with this module, or using the self-contained F<parl> utility
+on machines without PAR.pm installed.
 
 Note that self-containing scripts and executables created with F<par.pl>
 and F<pp> may also be used as F<.par> archives:
@@ -135,16 +135,16 @@ discussed on the mailing list.  Join us if you have an idea or two!
 
 =cut
 
-use vars qw(@PAR_INC);                  # explicitly stated PAR library files
+use vars qw(@PAR_INC);  # explicitly stated PAR library files
+use vars qw(%PAR_INC);  # sets {$par}{$file} for require'd modules
 use vars qw(@LibCache %LibCache);       # I really miss pseudohash.
+use vars qw($LastAccessedPAR);
 
 my $ver  = $Config::Config{version};
 my $arch = $Config::Config{archname};
 
-my $_reentrant;                         # flag to avoid recursive import
 sub import {
     my $class = shift;
-    return if !@_ and $_reentrant++;
 
     foreach my $par (@_) {
         if ($par =~ /[?*{}\[\]]/) {
@@ -158,14 +158,16 @@ sub import {
         push @PAR_INC, unpar($par, undef, undef, 1);
     }
 
+    return if $PAR::__import;
+    local $PAR::__import = 1;
+
     unshift @INC, \&find_par unless grep { $_ eq \&find_par } @INC;
 
     require PAR::Heavy;
     PAR::Heavy::_init_dynaloader();
 
     if (unpar($0)) {
-        $PAR::__reading = 1;
-        push @PAR_INC, $0;
+        push @PAR_INC, unpar($0, undef, undef, 1);
 
         my $zip = $LibCache{$0};
         my $member = $zip->memberNamed("script/main.pl")
@@ -186,24 +188,26 @@ sub import {
 
         _run_member($member);
     }
-
-    $_reentrant-- if !@_;
 }
 
 sub _run_member {
     my $member = shift;
+    my $clear_stack = shift;
     my ($fh, $is_new) = _tmpfile($member->crc32String . ".pl");
 
     if ($is_new) {
         my $file = $member->fileName;
-        print $fh "package main; shift \@INC;\n#line 1 \"$file\"\n";
+        print $fh "package main; shift \@INC;\n";
+        if (defined &Internals::PAR::CLEARSTACK and $clear_stack) {
+            print $fh "Internals::PAR::CLEARSTACK();\n";
+        }
+        print $fh "#line 1 \"$file\"\n";
         $member->extractToFileHandle($fh);
         seek ($fh, 0, 0);
     }
 
     unshift @INC, sub { $fh };
 
-    $PAR::__reading = 0;
     { do 'main'; die $@ if $@; exit }
 }
 
@@ -213,19 +217,35 @@ sub find_par {
     my $scheme;
     foreach (@PAR_INC ? @PAR_INC : @INC) {
         my $path = $_;
-        if (!@PAR_INC and $path and $path =~ m!//! and $scheme and $scheme =~ /^\w+$/) {
-            $path = "$scheme:$path";
-        }
-        else {
+        if ($[ < 5.008001) {
+            # reassemble from "perl -Ischeme://path" autosplitting
+            $path = "$scheme:$path" if !@PAR_INC
+                and $path and $path =~ m!//!
+                and $scheme and $scheme =~ /^\w+$/;
             $scheme = $path;
         }
-        my $rv = unpar($path, $file, $member_only, 1);
-        return $rv if defined($rv);
+        my $rv = unpar($path, $file, $member_only, 1) or next;
+        $PAR_INC{$path}{$file} = 1;
+        return $rv;
     }
 
     return;
 }
 
+sub reload_libs {
+    my @par_files = @_;
+    @par_files = sort keys %LibCache unless @par_files;
+
+    foreach my $par (@par_files) {
+        my $inc_ref = $PAR_INC{$par} or next;
+        delete $LibCache{$par};
+        foreach my $file (sort keys %$inc_ref) {
+            delete $INC{$file};
+            require $file;
+        }
+    }
+}
+
 sub read_file {
     my $file = pop;
 
@@ -248,22 +268,34 @@ sub unpar {
     my $zip = $LibCache{$par};
     my @rv = $par;
 
-    return if $PAR::__reading;
-    local $PAR::__reading = 1;
+    return if $PAR::__unpar;
+    local $PAR::__unpar = 1;
 
     unless ($zip) {
         if ($par =~ m!^\w+://!) {
             require File::Spec;
             require LWP::Simple;
-            $ENV{PAR_TEMP} ||= File::Spec->catdir(File::Spec->tmpdir, 'par');
-            mkdir $ENV{PAR_TEMP}, 0777;
+
+            # reflector support
+            $par .= "pm=$file" if $par =~ /[?&;]/;
+
+            $ENV{PAR_CACHE} ||= '_par';
+            mkdir $ENV{PAR_CACHE}, 0777;
+            if (!-d $ENV{PAR_CACHE}) {
+                $ENV{PAR_CACHE} = File::Spec->catdir(File::Spec->tmpdir, 'par');
+                mkdir $ENV{PAR_CACHE}, 0777;
+                return unless -d $ENV{PAR_CACHE};
+            }
 
             my $file = $par;
             if (!%escapes) {
                 $escapes{chr($_)} = sprintf("%%%02X", $_) for 0..255;
             }
-            $file =~ s/([^\w\.])/$escapes{$1}/g;
-            $file = File::Spec->catfile( $ENV{PAR_TEMP}, $file);
+            {
+                use bytes;
+                $file =~ s/([^\w\.])/$escapes{$1}/g;
+            }
+            $file = File::Spec->catfile( $ENV{PAR_CACHE}, $file);
             LWP::Simple::mirror( $par, $file );
             return unless -e $file;
             $par = $file;
@@ -278,12 +310,14 @@ sub unpar {
             return unless -f $par;
         }
 
-        require Compress::Zlib;
         require Archive::Zip;
-
         $zip = Archive::Zip->new;
         my $method = (ref($par) ? 'readFromFileHandle' : 'read');
-        return unless $zip->$method($par) == Archive::Zip::AZ_OK();
+
+        Archive::Zip::setErrorHandler(sub {});
+        my $rv = $zip->$method($par);
+        Archive::Zip::setErrorHandler(undef);
+        return unless $rv == Archive::Zip::AZ_OK();
 
         push @LibCache, $zip;
         $LibCache{$_[0]} = $zip;
@@ -294,11 +328,12 @@ sub unpar {
             next if $member->isDirectory;
             my $content = $member->contents();
             next unless $content =~ /^PK\003\004/;
-            local $PAR::__reading = 0;
             push @rv, unpar(\$content, undef, undef, 1);
         }
     }
 
+    $LastAccessedPAR = $zip;
+
     return @rv unless defined $file;
 
     my $member = $zip->memberNamed("lib/$file")
@@ -395,11 +430,11 @@ have sent helpful patches, ideas or comments.
 
 Autrijus Tang E<lt>autrijus@autrijus.orgE<gt>
 
-PAR has a mailing list, E<lt>par@perl.orgE<gt>, that you can write to;
-send an empty mail to E<lt>par-subscribe@perl.orgE<gt> to join the list
-and participate in the discussion.
+L<http://par.perl.org/> is the official PAR website.  You can write
+to the mailing list at E<lt>par@perl.orgE<gt>, or send an empty mail to
+E<lt>par-subscribe@perl.orgE<gt> to participate in the discussion.
 
-Please send bug reports to E<lt>bug-par@rt.cpan.orgE<gt>.
+Please submit bug reports to E<lt>bug-par@rt.cpan.orgE<gt>.
 
 =head1 COPYRIGHT
 
