@@ -1,11 +1,11 @@
-#!/usr/bin/perl
+#!/usr/local/bin/perl
 # $File: //member/autrijus/PAR/script/par.pl $ $Author: autrijus $
-# $Revision: #67 $ $Change: 7148 $ $DateTime: 2003/07/27 07:21:03 $ vim: expandtab shiftwidth=4
+# $Revision: #80 $ $Change: 8491 $ $DateTime: 2003/10/19 15:19:48 $ vim: expandtab shiftwidth=4
 
 package __par_pl;
 
 # --- This script must not use any modules at compile time ---
-#use strict;
+# use strict;
 
 =head1 NAME
 
@@ -23,11 +23,12 @@ To make a I<PAR distribution> from a CPAN module distribution:
 
 To manipulate a I<PAR distribution>:
 
-    % par.pl -i Foo-0.01-i386-freebsd-5.8.0.par     # install
-    % par.pl -i http://example.com/foo.par          # also works
-    % par.pl -u Foo-0.01-i386-freebsd-5.8.0.par     # uninstall
-    % par.pl -s Foo-0.01-i386-freebsd-5.8.0.par     # sign
-    % par.pl -v Foo-0.01-i386-freebsd-5.8.0.par     # verify
+    % par.pl -i Foo-0.01-i386-freebsd-5.8.0.par	# install
+    % par.pl -i http://foo.com/Foo-0.01		# auto-appends archname + perlver
+    % par.pl -i cpan://AUTRIJUS/PAR-0.74	# uses CPAN author directory
+    % par.pl -u Foo-0.01-i386-freebsd-5.8.0.par # uninstall
+    % par.pl -s Foo-0.01-i386-freebsd-5.8.0.par # sign
+    % par.pl -v Foo-0.01-i386-freebsd-5.8.0.par # verify
 
 To use F<Hello.pm> from F<./foo.par>:
 
@@ -149,6 +150,19 @@ followed by a 8-bytes magic string: "C<\012PAR.pm\012>".
 
 =cut
 
+my ($tmpfile, @tmpfiles);
+END { unlink @tmpfiles if @tmpfiles }
+
+BEGIN {
+    Internals::PAR::BOOT() if defined &Internals::PAR::BOOT;
+
+    eval {
+
+if ($ENV{PAR_ARGV_0}) {
+    @ARGV = map $ENV{"PAR_ARGV_$_"}, (1 .. $ENV{PAR_ARGC} - 1);
+    $0 = $ENV{PAR_ARGV_0};
+}
+
 $ENV{PAR_CLEARTEMP} = 1 unless exists $ENV{PAR_CLEARTEMP};
 my $quiet = !$ENV{PAR_DEBUG};
 
@@ -175,6 +189,8 @@ elsif (!-s $0) {
 # Magic string checking and extracting bundled modules {{{
 my ($start_pos, $data_pos);
 {
+    local $SIG{__WARN__} = sub {};
+
     # Check file type, get start of data section {{{
     open _FH, $0 or last;
     binmode(_FH);
@@ -221,9 +237,9 @@ my ($start_pos, $data_pos);
         }
         elsif ( $fullname =~ m|^/?shlib/| and defined $ENV{PAR_TEMP} ) {
             # should be moved to _tempfile()
-            $filename = "$ENV{PAR_TEMP}/$basename$ext";
-            print "SHLIB: $filename\n";
-            open $out, '>', $filename or die $!;
+            my $filename = "$ENV{PAR_TEMP}/$basename$ext";
+            outs("SHLIB: $filename\n");
+            open my $out, '>', $filename or die $!;
             binmode($out);
             print $out $buf;
             close $out;
@@ -343,6 +359,8 @@ if (!$start_pos or ($ARGV[0] eq '--par-options' && shift)) {
         shift(@ARGV);
 
         if (my $cmd = $dist_cmd{$1}) {
+            delete $ENV{'PAR_TEMP'};
+            init_inc();
             require PAR::Dist;
             &{"PAR::Dist::$cmd"}() unless @ARGV;
             &{"PAR::Dist::$cmd"}($_) for @ARGV;
@@ -388,6 +406,7 @@ if ($out) {
     if ($bundle) {
         require PAR::Heavy;
         PAR::Heavy::_init_dynaloader();
+        init_inc();
         require_modules();
 
         my @inc = sort {
@@ -400,6 +419,8 @@ if ($out) {
              $_ ne $Config::Config{privlibexp});
         } @INC;
 
+        if ($^O eq 'MSWin32') { s{\\}{/}g for @inc }
+
         my %files;
         /^_<(.+)$/ and $files{$1}++ for keys %::;
         $files{$_}++ for values %INC;
@@ -415,7 +436,7 @@ if ($out) {
                     $file = $_;
                     last;
                 }
-                elsif (/^(\Q$dir\E\/(.*[^Cc]))\Z/) {
+                elsif (/^(\Q$dir\E\/(.*[^Cc]))\Z/i) {
                     ($file, $name) = ($1, $2);
                     last;
                 }
@@ -486,31 +507,14 @@ if ($out) {
 {
     last unless defined $start_pos;
 
-    # Set up fake IO::File routines to point into the PAR subfile {{{
-    require IO::File;
-    my $fh = IO::File->new($0);
-    my $seek_ref  = $fh->can('seek');
-    my $tell_ref  = $fh->can('tell');
-
-    *{'IO::File::seek'} = sub {
-        return $seek_ref->(@_) unless $PAR::__reading;
-        my ($fh, $pos, $whence) = @_;
-        $pos += $start_pos if $whence == 0;
-        $seek_ref->($fh, $pos, $whence);
-    };
-    *{'IO::File::tell'} = sub {
-        return $tell_ref->(@_) unless $PAR::__reading;
-        return $tell_ref->(@_) - $start_pos;
-    };
-    # }}}
-
     # Now load the PAR file and put it into PAR::LibCache {{{
     require PAR;
     PAR::Heavy::_init_dynaloader();
-    require Archive::Zip;
 
-    local $PAR::__reading = 1;
+    require Archive::Zip;
     my $zip = Archive::Zip->new;
+    my $fh = IO::File->new;
+    $fh->fdopen(fileno(_FH), 'r') or die "$!: $@";
     $zip->readFromFileHandle($fh) == Archive::Zip::AZ_OK() or die "$!: $@";
 
     push @PAR::LibCache, $zip;
@@ -522,7 +526,14 @@ if ($out) {
     foreach my $member ( $zip->members ) {
         next if $member->isDirectory or !$ENV{PAR_TEMP};
         my $member_name = $member->fileName;
-        next unless $member_name =~ m{^/?shlib/(?:$Config::Config{version}/)?(?:$Config::Config{archname}/)?([^/]+)$};
+        next unless $member_name =~ m{
+            ^
+            /?shlib/
+            (?:$Config::Config{version}/)?
+            (?:$Config::Config{archname}/)?
+            ([^/]+)
+            $
+        }x;
         my $extract_name = $1;
         my $dest_name = File::Spec->catfile($ENV{PAR_TEMP}, $extract_name);
         $member->extractToFileNamed($dest_name);
@@ -538,7 +549,7 @@ unless ($PAR::LibCache{$0}) {
 Usage: $0 [ -Alib.par ] [ -Idir ] [ -Mmodule ] [ src.par ] [ program.pl ]
        $0 [ -B|-b ] [-Ooutfile] src.par
 .
-    $0 = shift(@ARGV)
+    $0 = shift(@ARGV);
 }
 # }}}
 
@@ -587,7 +598,6 @@ sub tmpdir {
     return $tmpdir;
 }
 
-my ($tmpfile, @tmpfiles);
 sub _tempfile {
     my ($ext, $crc) = @_;
     my ($fh, $filename);
@@ -618,7 +628,6 @@ sub _tempfile {
     binmode($fh);
     return ($fh, $filename);
 }
-END { unlink @tmpfiles if @tmpfiles }
 
 sub outs { warn("@_\n") unless $quiet }
 
@@ -626,6 +635,10 @@ sub pod_strip {
     my ($pl_text, $filename) = @_;
 
     local $^W;
+
+    my $data = '';
+    $data = $1 if $pl_text =~ s/((?:^__DATA__$).*)//ms;
+
     my $line = 1;
     if ($pl_text =~ /^=(?:head\d|pod|begin|item|over|for|back|end)\b/) {
         $pl_text = "\n$pl_text";
@@ -647,7 +660,15 @@ sub pod_strip {
         if length $filename;
     $pl_text =~ s/^#line 1 (.*\n)(#!.*\n)/$2#line 2 $1/g;
 
-    return $pl_text;
+    return $pl_text . $data;
+}
+
+sub init_inc {
+    require Config;
+    push @INC, grep defined, map $Config::Config{$_}, qw(
+        archlibexp privlibexp sitearchexp sitelibexp
+        vendorarchexp vendorlibexp
+    );
 }
 
 ########################################################################
@@ -664,7 +685,15 @@ die qq(Can't open perl script "$0": No such file or directory\n)
 
 do $0;
 die $@ if $@;
-exit;
+
+};
+
+$::__ERROR = $@ if $@;
+}
+
+die $::__ERROR if $::__ERROR;
+
+1;
 
 =head1 SEE ALSO
 
@@ -674,11 +703,11 @@ L<PAR>, L<PAR::Dist>, L<parl>, L<pp>
 
 Autrijus Tang E<lt>autrijus@autrijus.orgE<gt>
 
-PAR has a mailing list, E<lt>par@perl.orgE<gt>, that you can write to;
-send an empty mail to E<lt>par-subscribe@perl.orgE<gt> to join the list
-and participate in the discussion.
+L<http://par.perl.org/> is the official PAR website.  You can write
+to the mailing list at E<lt>par@perl.orgE<gt>, or send an empty mail to
+E<lt>par-subscribe@perl.orgE<gt> to participate in the discussion.
 
-Please send bug reports to E<lt>bug-par@rt.cpan.orgE<gt>.
+Please submit bug reports to E<lt>bug-par@rt.cpan.orgE<gt>.
 
 =head1 COPYRIGHT
 
